#version 120
// (C) 2009 Dmytry Lavrov. All rights reserved.

uniform float brightness;

uniform mat4 color_matrix;

uniform float wiggle_freq;
uniform float wiggle_amplitude;
uniform float wiggle_phase;
uniform float turn;

// Approximate the elliptic integral (constant length backbone)
float approx_elliptic_integral_offset(float alpha){
	float a = 2.0 * wiggle_amplitude;
	float b = wiggle_freq + 1E-20;
	float c = 1.0 / sqrt(a * a * b * b + 1.0);
	float q = 0.5 * (1.0 - c);
	float p = 0.5 * (c + 1.0);
	float s_a = 0.5 * q / b;
	//wiggled_pos.z = wiggled_pos.z * p + sin(2.0 * alpha + wiggle_phase) * s_a;
	return sin(2.0 * alpha + wiggle_phase) * s_a;

	// Approximate the elliptic integral, weirder but better looking approach.
	// wiggled_pos.xyz += wiggle_freq.xyz * (sin(2. * alpha + wiggle_phase) * s_a / b);
}

void main(void)
{
	float alpha=-gl_Vertex.z*wiggle_freq;
	vec4 wiggled_pos=gl_Vertex;

	float scale=gl_Vertex.x*gl_Vertex.x/sqrt(gl_Vertex.x*gl_Vertex.x+0.5);
	wiggled_pos.y+=wiggle_amplitude*sin(alpha+wiggle_phase)*scale;

	wiggled_pos.z-=approx_elliptic_integral_offset(alpha)*scale;

	// turns:
	if (abs(turn) > 1E-5)
	{
		float i_turn = 1.0 / turn;
		float turn_a = wiggled_pos.z * turn;
		if (turn > 0.0)
		{
			wiggled_pos.z = sin(turn_a) * (i_turn + wiggled_pos.y);
			wiggled_pos.y = cos(turn_a) * (i_turn + wiggled_pos.y) - i_turn;
		}
		else
		{
			wiggled_pos.z = sin(-turn_a) * (-i_turn - wiggled_pos.y);
			wiggled_pos.y = -i_turn - cos(-turn_a) * (-i_turn - wiggled_pos.y);
		}
	}

	vec4 pos = gl_ModelViewProjectionMatrix * wiggled_pos;
  vec3 delta=(gl_ModelViewMatrix*gl_Vertex).xyz;
  float l=dot(delta,delta);
  gl_Position=pos;
  vec3 c=vec3(color_matrix*wiggled_pos);
  c=abs(2.0*fract(c)-vec3(1.0,1.0,1.0));/// triangle wave
  gl_FrontColor=vec4(c*(brightness/l), 1.0);
}



/*

uniform vec3  wiggle_freq;
uniform vec3  wiggle_dir;
uniform float wiggle_phase;
uniform float turn;

const vec3 lightPos = vec3(0., 200., 0.);

void main(void)
{
	float a = 2.0 * length(wiggle_dir);
	float b = length(wiggle_freq) + 1E-20;
	float c = 1.0 / sqrt(a * a * b * b + 1.);
	float q = 0.5 * (1. - c);
	float p = 0.5 * (c + 1.);
	float s_a = 0.5 * q / b;

	float alpha = dot(vec3(gl_Vertex), wiggle_freq);

	vec4 new_wiggle_dir = vec4(wiggle_dir.x, wiggle_dir.y, wiggle_dir.z, 0.);

	vec4 wiggled_pos = gl_Vertex + new_wiggle_dir * sin(alpha + wiggle_phase);

	// Approximate the elliptic integral
	wiggled_pos.x = wiggled_pos.x * p + sin(2. * alpha + wiggle_phase) * s_a;

	// Approximate the elliptic integral, weirder but better looking.
	// wiggled_pos.xyz += wiggle_freq.xyz * (sin(2. * alpha + wiggle_phase) * s_a / b);
	// turns:
	if (abs(turn) > 1E-5)
	{
		float i_turn = 1. / turn;
		float turn_a = wiggled_pos.x * turn;

		if (turn > 0.)
		{
			wiggled_pos.x = sin(turn_a) * (i_turn + wiggled_pos.z);
			wiggled_pos.z = cos(turn_a) * (i_turn + wiggled_pos.z) - i_turn;
		}
		else
		{
			wiggled_pos.x = sin(-turn_a) * (-i_turn - wiggled_pos.z);
			wiggled_pos.z = -i_turn - cos(-turn_a) * (-i_turn - wiggled_pos.z);
		}
	}

	gl_Position = gl_ModelViewProjectionMatrix * wiggled_pos;
	gl_FrontSecondaryColor = gl_SecondaryColor;
	gl_TexCoord[0] = gl_MultiTexCoord0;

	// Apply diffuse lighting using the non-transformed normals
	// TODO: transform the normals as well
	vec3 N = normalize(gl_NormalMatrix * gl_Normal);
	vec4 V = gl_ModelViewMatrix * wiggled_pos;
	vec3 L = normalize(lightPos - V.xyz);

	float NdotL = max(0.0, dot(N, L));

	gl_FrontColor = gl_Color * vec4(NdotL, NdotL, NdotL, 1.0);
}


*/
